Class: Fbe::Conclude
- Inherits:
-
Object
- Object
- Fbe::Conclude
- Defined in:
- lib/fbe/conclude.rb
Overview
A concluding block.
You may want to use this class when you want to go through a number of facts in the factbase, applying certain algorithm to each of them and possibly creating new facts from them.
For example, you want to make a new good fact for every bad fact found:
require 'fbe/conclude'
conclude do
on '(exist bad)'
follow 'when'
draw on |n, b|
n.good = 'yes!'
end
end
This snippet will find all facts that have bad property and then create new facts, letting the block in the #draw deal with them.
- Author
-
Yegor Bugayenko ([email protected])
- Copyright
-
Copyright © 2024-2025 Zerocracy
- License
-
MIT
Instance Method Summary collapse
-
#consider {|Factbase::Fact| ... } ⇒ Integer
Take every fact, allowing the given block to process it.
-
#draw {|Array<Factbase::Fact,Factbase::Fact>| ... } ⇒ Integer
Create new fact from every fact found by the query.
-
#follow(props) ⇒ nil
Set the list of properties to copy from the facts found to new facts.
-
#initialize(fb:, judge:, global:, options:, loog:) ⇒ Conclude
constructor
Ctor.
-
#on(query) ⇒ nil
Set the query that should find the facts in the factbase.
-
#quota_aware ⇒ nil
Make this block aware of GitHub API quota.
-
#timeout(sec) ⇒ nil
Make sure this block runs for less than allowed amount of seconds.
Constructor Details
#initialize(fb:, judge:, global:, options:, loog:) ⇒ Conclude
Ctor.
56 57 58 59 60 61 62 63 64 65 66 |
# File 'lib/fbe/conclude.rb', line 56 def initialize(fb:, judge:, global:, options:, loog:) @fb = fb @judge = judge @loog = loog = @global = global @query = nil @follows = [] @quota_aware = false @timeout = 60 end |
Instance Method Details
#consider {|Factbase::Fact| ... } ⇒ Integer
Take every fact, allowing the given block to process it.
For example, you want to add when property to every fact:
require 'fbe/conclude'
conclude do
on '(always)'
consider on |f|
f.when = Time.new
end
end
149 150 151 152 153 154 |
# File 'lib/fbe/conclude.rb', line 149 def consider(&) roll do |_fbt, a| yield a nil end end |
#draw {|Array<Factbase::Fact,Factbase::Fact>| ... } ⇒ Integer
Create new fact from every fact found by the query.
For example, you want to conclude a reward from every win fact:
require 'fbe/conclude'
conclude do
on '(exist win)'
follow 'win when'
draw on |n, w|
n.reward = 10
end
end
This snippet will find all facts that have win property and will create new facts for all of them, passing them one by one in to the block of the draw, where n would be the new created fact and the w would be the fact found.
127 128 129 130 131 132 133 |
# File 'lib/fbe/conclude.rb', line 127 def draw(&) roll do |fbt, a| n = fbt.insert fill(n, a, &) n end end |
#follow(props) ⇒ nil
Set the list of properties to copy from the facts found to new facts.
103 104 105 |
# File 'lib/fbe/conclude.rb', line 103 def follow(props) @follows = props.strip.split.compact end |
#on(query) ⇒ nil
Set the query that should find the facts in the factbase.
94 95 96 97 |
# File 'lib/fbe/conclude.rb', line 94 def on(query) raise 'Query is already set' unless @query.nil? @query = query end |
#quota_aware ⇒ nil
Make this block aware of GitHub API quota.
When the quota is reached, the loop will gracefully stop to avoid hitting GitHub API rate limits. This helps prevent interruptions in long-running operations.
75 76 77 |
# File 'lib/fbe/conclude.rb', line 75 def quota_aware @quota_aware = true end |
#timeout(sec) ⇒ nil
Make sure this block runs for less than allowed amount of seconds.
When the quota is reached, the loop will gracefully stop to avoid. This helps prevent interruptions in long-running operations.
86 87 88 |
# File 'lib/fbe/conclude.rb', line 86 def timeout(sec) @timeout = sec end |